﻿Imports System.Net.Sockets
Imports FServerSupport.PLCNetwork

' A sample project that connects to an F-server or TLServer via TCP/IP to send and receive hostlink 
' commands.
'
' This project refereces the DLL "FServerSupport.DLL" which is included in the "ConnectFServer" 
' project folder. Your program may need to click on "Project->Add Reference", then click on 
' the "Browse" tab and browse to find and select the "FServerSupport.DLL" file.
' To run the .exe file generated by your Visual Basic Express compiler, you should copy the "FServerSupport.DLL" file
' to the same folder as the .exe file.

Public Class Form1
    Private Sub Button1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button1.Click
        Try
            TextBox1.AppendText("Contacting PLC Webserver: " & IPAddr.Text & ":" & PortNo.Text & vbCrLf)
            Dim client1 As New TcpClient(IPAddr.Text, PortNo.Text)
            Dim stream1 As NetworkStream = client1.GetStream()
            Dim command, response As String
            stream1.ReadTimeout = 1000

            Dim Message = ConnectToServer(stream1, username.Text, password.Text)
            TextBox1.AppendText("Received:" & Message & vbCrLf)

            If Message.Length >= 4 Then
                If Message.Substring(0, 4) = "OK-1" Then
                    command = appendFCS(1, "RI00")         ' Enter your hostlink commands for testing
                    response = sendCommand(stream1, command)
                    TextBox1.AppendText("Sent:" & command & vbCrLf)
                    TextBox1.AppendText("Received:" & response & vbCrLf)

                    command = appendFCS(1, "IR")         ' Enter your hostlink commands for testing
                    response = sendCommand(stream1, command)
                    TextBox1.AppendText("Sent:" & command & vbCrLf)
                    TextBox1.AppendText("Received:" & response & vbCrLf)
                End If
            End If
            stream1.Close()
            client1.Close()
        Catch ex As SocketException
            Console.WriteLine("SocketException: {0}", ex)
            TextBox1.AppendText(ex.ToString)
        End Try
    End Sub

    Public Function ConnectToServer(ByRef stream As [NetworkStream], _
                                  ByVal usr As [String], ByVal pw As [String]) As String
        Try
            ' Translate the login data into a Byte array.
            Dim plcnet = New FServerSupport.PLCNetwork

            Dim str1 As String = "GET /TLcomm" & vbCrLf & usr & vbCrLf & plcnet.encrypt(pw) & vbCrLf
            Dim OutData As [Byte]() = System.Text.Encoding.ASCII.GetBytes(str1)
            Dim InData = New [Byte](127) {}

            ' Send the message to the connected TcpServer. 
            stream.Write(OutData, 0, OutData.Length)
            Dim responseData As [String]
            Dim byteCount As Int32 = stream.Read(InData, 0, InData.Length)
            responseData = System.Text.Encoding.ASCII.GetString(InData, 0, byteCount)
            If stream.DataAvailable Then
                byteCount = stream.Read(InData, 0, InData.Length) ' clear input buffer
                Dim responseData2 = System.Text.Encoding.ASCII.GetString(InData, 0, byteCount)
            End If
            ConnectToServer = responseData
            Exit Function
        Catch e As System.IO.IOException
            Dim errMsg = "No response from server"
            Console.WriteLine(errMsg)
            ConnectToServer = errMsg
            Exit Function
        End Try
        ConnectToServer = ""
    End Function
    Public Function sendCommand(ByRef stream As [NetworkStream], ByVal command As [String]) As String
        Dim I As Int32
        Dim response As String
        For I = 1 To 3
            response = sendOnce(stream, command)
            If (response.Length > 5) Then ' check if the response is of the same header as the command
                If response.Substring(0, 5) = command.Substring(0, 5) _
                    And verifyFCS(response) Then
                    sendCommand = response
                    Exit Function
                End If
            End If
        Next
        sendCommand = ""
    End Function

    Public Function sendOnce(ByRef stream As [NetworkStream], ByVal command As [String]) As String
        Try
            Dim InData = New [Byte](127) {}
            Dim InData2 = New Byte(127) {}
            Dim OutData = System.Text.Encoding.ASCII.GetBytes(command & vbCrLf)  ' Append CR to command string
            stream.Write(OutData, 0, OutData.Length)               ' send out the command string to the server

            Dim byteCount = stream.Read(InData, 0, InData.Length) ' read the response string from the server
            Dim byteCount2 As Integer
            If (byteCount) Then            ' check if InData is terminated by CR-LF
                If InData(byteCount - 1) <> &HA Then  ' if not it means CR-LF may be in the next packet
                    byteCount2 = stream.Read(InData2, 0, InData2.Length) ' read the next response string from the server
                End If
                For i = 0 To byteCount2 - 1   ' combine the next packet to the first packet
                    InData(byteCount + i) = InData2(i)
                Next
                sendOnce = System.Text.Encoding.ASCII.GetString(InData, 0, byteCount + byteCount2)
                Exit Function
            Else
                sendOnce = ""    ' blank string means no response
                Exit Function
            End If
        Catch e As System.IO.IOException
            Dim errMsg = "No response from server"
            Console.WriteLine(errMsg)
            sendOnce = errMsg
        End Try
    End Function

    ' This function append the ID and the FCS to the input string "cmd" and return
    ' the resultant string. cmd should not include the "*" character

    Public Function appendFCS(ByRef ID As Short, ByRef cmd As String) As String
        Dim IDString As String
        Dim FCSString As String
        IDString = Hex(ID)
        If Len(IDString) = 1 Then IDString = "0" & IDString
        cmd = "@" & IDString & cmd
        'UPGRADE_WARNING: Couldn't resolve default property of object computeFCS(). Click for more: 'ms-help://MS.VSCC.v90/dv_commoner/local/redirect.htm?keyword="6A50421D-15FE-4896-8A1B-2EC21E9037B2"'
        FCSString = Hex(computeFCS(cmd))
        If Len(FCSString) = 1 Then FCSString = "0" & FCSString
        cmd = cmd & FCSString & "*"
        appendFCS = cmd ' returned string now contains @ID and FCS+"*"
    End Function

    ' This function compute the FCS for the given input string "cmd" and return the
    ' computed FCS as an integer. This function may be called for verifying whether
    ' the received response string is error free.

    Public Function computeFCS(ByRef cmd As String) As Integer
        Dim result As Short
        Dim length As Short
        Dim I As Short
        length = Len(cmd)
        result = Asc(Mid(cmd, 1, 1))
        For I = 2 To length
            result = result Xor Asc(Mid(cmd, I, 1))
        Next
        computeFCS = result
    End Function

    Public Function verifyFCS(ByRef str As String) As Boolean
        Dim newstr = str.Substring(0, str.IndexOf("*"c))
        Dim FCS As Integer = Val("&h" & newstr.Substring(newstr.Length - 2, 2))
        Dim computedFCS = computeFCS(newstr.Substring(0, newstr.Length - 2))
        If FCS = computedFCS Then
            verifyFCS = True
        Else
            verifyFCS = False
        End If
    End Function
End Class
